/*#############################################################
File Name       : ex1_HDJD_S822_vcp.c
Author          : Grant Phillips
Date Modified   : 15/04/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F3-Discovery

Description     : Example program that demonstrates the use
									of the HDJD-S822 Color Sensor (https://www.
									sparkfun.com/products/10904).
									
									The output is displayed on a PC using the
									Virtual COM Port (VCP).  See the VCP examples
									for an explanation of the use of this feature.
									
Requirements    : * STM32F3-Discovery Board
									* Make sure the VCP_F3.c file is in the 
                    same folder than the project
              
Circuit         : * The HDJD-S822 Color Sensor is connected 
										as follows:
										GSR1									-	PC7
										GSR0									-	PC6
										GSG1									-	PC5
										GSG0									-	PC4
										GSB1									-	PC3
										GSB0									-	PC2
										5V										-	5V
										GND										-	GND
										LED										-	PC1
										VR										-	PD10
										VG										-	PD11
										VB										-	PD12
									* Mini-USB cable from the User USB port on the
										STM32F3-Discovery to a USB port on a PC
									
See the STM32F3-Discovery User Manual (UM1570) for the block 
diagram of the STM32F303VCT6 processor (p13), a summary of
the GPIO connections (p21-29) and the schematic diagram (p31-34)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "main.h"																								//main library to include for device drivers, peripheral drivers, etc.
#include "VCP_F3.c"																							//for the UART4 functions on the STM32F3-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void InitSystem(void);																					//prototype for the user function to initialize the system

/* #global variables -----------------------------------------*/
RCC_ClocksTypeDef 			RCC_Clocks;															//structure used for setting up the SysTick Interrupt

__IO uint16_t ADC3_val[3];																			//array to hold the 4 converted values 
																																//ADC3_val[0] - PD10, ADC3_val[1] - PD11, ADC3_val[2] - PD12


// Unused global variables that have to be included to ensure correct compiling */
// ###### DO NOT CHANGE ######
// ===============================================================================
__IO uint32_t TimingDelay = 0;																	//used with the Delay function
__IO uint8_t DataReady = 0;
__IO uint32_t USBConnectTimeOut = 100;
__IO uint32_t UserButtonPressed = 0;
__IO uint8_t PrevXferComplete = 1;
// ===============================================================================



int main(void)
{
	char buf[40];																									//string variable of 40 characters
	uint32_t RED, GREEN, BLUE;
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config
	
	/* initialize the IO ports and Analogs */
	InitSystem();							
	
	/* initialize the Virtual COM Port */
	VCP_ResetPort();																							//pull the USB D+ line low to disconnect the USB device from the PC
	Delay(500);																										//allow a few milliseconds before reconnecting
	VCP_Init();																										//initialize the STM32F3-Discovery as a Virtual COM Port - device gets re-detected by the PC
	Delay(1000);																									//wait for USB to be connected
	
	/* set the RED gain to 0 
	GSR1	GSR0	GAIN
	0			0			0
	0			1			1
	1			0			2
	1			1			3							
	
	Refer to page 6 - 7 of the datasheet to see what effect
	the gain will have on the result*/
	GPIO_WriteBit(GPIOC, GPIO_Pin_7, Bit_RESET);		//GSR1
	GPIO_WriteBit(GPIOC, GPIO_Pin_6, Bit_RESET);		//GSR0
	
	/* set the GREEN gain to 0 */
	GPIO_WriteBit(GPIOC, GPIO_Pin_5, Bit_RESET);		//GSG1
	GPIO_WriteBit(GPIOC, GPIO_Pin_4, Bit_RESET);		//GSG0
	
	/* set the BLUE gain to 0 */
	GPIO_WriteBit(GPIOC, GPIO_Pin_3, Bit_RESET);		//GSB1
	GPIO_WriteBit(GPIOC, GPIO_Pin_2, Bit_RESET);		//GSB0
	
	/* turn on the LED on the HDJD-S822 */
	GPIO_WriteBit(GPIOC, GPIO_Pin_1, Bit_SET);		//LED
	
	
	/* Main program loop */
  while (1)
  {
		// On page 7 of the datasheet, there is a graph of the 
		// spectral responsivity of the chip.  Scaling factors were
		// selected based on this graph so that the gain of each 
		// color is closer to being equal 
		RED = ADC3_val[0] * 10;
		GREEN = ADC3_val[1] * 14;
		BLUE = ADC3_val[2] * 17;
		
		/* display the color results */
		sprintf(buf, "R:%4d  G:%4d  B:%4d\n", RED, GREEN, BLUE);
		VCP_PutStr(buf);
		
    Delay(200);
  }	
}



void InitSystem(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;											//structure used for setting up a GPIO port
	ADC_InitTypeDef   			ADC_InitStructure;
	ADC_CommonInitTypeDef 	ADC_CommonInitStructure;
	DMA_InitTypeDef   			DMA_InitStructure;											//structure for setting up the DMA
	__IO unsigned int ADC_calibration_value;												//variable for storing the calibration value
	
	// ================== Initialize the IO Ports ==================
	
	/* GPIOC Periph clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);							//enable the AHB bus to use GPIOC
  
  /* Configure PC7 - PC1 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_6 | GPIO_Pin_5 | GPIO_Pin_4 | GPIO_Pin_3 | GPIO_Pin_2 | GPIO_Pin_1;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;								
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;							
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;							
  GPIO_Init(GPIOC, &GPIO_InitStructure);			

	
	
	
	
	// ================== Initialize the Analogs ==================	
	/* The DMA Request mapping for ADC3 is on DMA2 channel 5, see p.160 in the STMF30x Reference Manual */
	
  /* Enable ADC3, DMA2 and GPIOC clock */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_ADC34 | RCC_AHBPeriph_DMA2 | RCC_AHBPeriph_GPIOD, ENABLE);
	
  /* DMA2 Channel5 Config */
	DMA_InitStructure.DMA_PeripheralBaseAddr = ((uint32_t)&ADC3->DR);						//specify the base address of the peripheral (ADC3) to connect to the DMA2 channel 5
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)&ADC3_val[0]; 							//specify the base address of the result array
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;														
  DMA_InitStructure.DMA_BufferSize = 3;																				//=2 because we are using 2 channels and each channel is stored in an array element
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;				
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;											//automatically increase the element number for each channel conversion
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;	//16bit size
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;					//16bit size
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;															//after all the channels are read by the DMA, start again
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMA2_Channel5, &DMA_InitStructure);
	
  /* DMA2 Channel5 enable */
  DMA_Cmd(DMA2_Channel5, ENABLE);

	/* Configure ADC3_IN7, ADC3_IN8, ADC3_IN9 as analog inputs */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
  GPIO_Init(GPIOD, &GPIO_InitStructure);
 
  /* ADC Common Init */
  ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_CommonInitStructure.ADC_Clock = ADC_Clock_SynClkModeDiv2;
  ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled;
	ADC_CommonInitStructure.ADC_DMAMode = ADC_DMAMode_Circular ;
	ADC_CommonInitStructure.ADC_TwoSamplingDelay = 10;															//insert a small delay between channel samples (10 cycles delay)
  ADC_CommonInit(ADC3, &ADC_CommonInitStructure);
 
  /* ADC3 regular configuration */
  ADC_InitStructure.ADC_ContinuousConvMode = ADC_ContinuousConvMode_Enable;				
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b; 													
  ADC_InitStructure.ADC_ExternalTrigConvEvent = ADC_ExternalTrigConvEvent_0;      
  ADC_InitStructure.ADC_ExternalTrigEventEdge = ADC_ExternalTrigEventEdge_None;		
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;													
  ADC_InitStructure.ADC_OverrunMode = ADC_OverrunMode_Disable;   									
  ADC_InitStructure.ADC_AutoInjMode = ADC_AutoInjec_Disable;  										
  ADC_InitStructure.ADC_NbrOfRegChannel = 3;																			//using 3 channels on ADC3
  ADC_Init(ADC3, &ADC_InitStructure);																							
	
  /* ADC1 regular channel configuration *************************************/
  ADC_RegularChannelConfig(ADC3, ADC_Channel_7, 1, ADC_SampleTime_7Cycles5);			//ADC3_IN7 will be read 1st = 1
  ADC_RegularChannelConfig(ADC3, ADC_Channel_8, 2, ADC_SampleTime_7Cycles5);	
  ADC_RegularChannelConfig(ADC3, ADC_Channel_9, 3, ADC_SampleTime_7Cycles5);	
	//ADC_RegularChannelConfig(ADC3, ADC_Channel_10, 4, ADC_SampleTime_7Cycles5);
 
  /* Enable ADC3 DMA */
  ADC_DMACmd(ADC3, ENABLE);
	
  /* Enable ADC3 */
  ADC_Cmd(ADC3, ENABLE);
	
	/* ADC DMA request in circular mode */
	ADC_DMAConfig(ADC3, 2);
	
  /* wait for ADRDY */
  while(!ADC_GetFlagStatus(ADC3, ADC_FLAG_RDY));
  
  /* Start ADC1 Software Conversion */ 
  ADC_StartConversion(ADC3); 
}





// -------------------------------------------------------------------------------

// Function to insert a timing delay of nTime
// ###### DO NOT CHANGE ######
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

// Function to Decrement the TimingDelay variable.
// ###### DO NOT CHANGE ######
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

// Unused functions that have to be included to ensure correct compiling
// ###### DO NOT CHANGE ######
// =======================================================================
uint32_t L3GD20_TIMEOUT_UserCallback(void)
{
  return 0;
}

uint32_t LSM303DLHC_TIMEOUT_UserCallback(void)
{
  return 0;
}
// =======================================================================



