/*#############################################################
File Name       : ex1_MaxSonarEZ0_measure_vcp.c
Author          : Grant Phillips
Date Modified   : 12/05/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F3-Discovery

Description     : Example program that demonstrates the use of
                  a MaxSonar EZ0 Sonar Range Finder.    

									The output is displayed on a PC using the
									Virtual COM Port (VCP).  See the VCP examples
									for an explanation of the use of this feature.
									
Requirements    : * STM32F3-Discovery Board
									* Make sure the VCP_F3.c file is in the 
                    same folder than the project
              
Circuit         : * Mini-USB cable from the User USB port on the
										STM32F3-Discovery to a USB port on a PC
									* The MaxSonar EZ0 Sonar Range Finder connected 
										as:
										GND		-		GND
										VCC		-		+3V
										PW		-		PB3
										
										
																		*NOTE*
									Servo1 - 4 will now not be availble due to it
									sharing resources with the driver for the 
									HC-SR04.  Servo5 - 12 will still be available.

See the STM32F3-Discovery User Manual (UM1570) for the block 
diagram of the STM32F303VCT6 processor (p13), a summary of
the GPIO connections (p21-29) and the schematic diagram (p31-34)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "main.h"																								//main library to include for device drivers, peripheral drivers, etc.
#include "VCP_F3.c"																							//for the Virtual COM Port functions on the STM32F3-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void MaxSonarEZ0_Init(void);																		//function prototype for a user funtion void MaxSonarEZ0_Init();

/* #global variables -----------------------------------------*/
GPIO_InitTypeDef        GPIO_InitStructure;											//structure used for setting up a GPIO port
RCC_ClocksTypeDef 			RCC_Clocks;															//structure used for setting up the SysTick Interrupt



// Unused global variables that have to be included to ensure correct compiling */
// ###### DO NOT CHANGE ######
// ===============================================================================
__IO uint32_t TimingDelay = 0;																	//used with the Delay function
__IO uint8_t DataReady = 0;
__IO uint32_t USBConnectTimeOut = 100;
__IO uint32_t UserButtonPressed = 0;
__IO uint8_t PrevXferComplete = 1;
// ===============================================================================



int main(void)
{
	char buf[40];																				//string variable of 40 characters   
  uint32_t distance;																	//declase a 32-bit unsigned int to hold the distance value
	float mm;																						//float variable to hold the distance value in mm
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																					//will end up in this infinite loop if there was an error with Systick_Config
	
	MaxSonarEZ0_Init();																	//initialize the MaxSonar EZ0 Sonar Range Finder - see the function definition below
	
	VCP_ResetPort();																		//pull the USB D+ line low to disconnect the USB device from the PC
	Delay(500);																					//allow a few milliseconds before reconnecting
	VCP_Init();																					//initialize the STM32F3-Discovery as a Virtual COM Port - device gets re-detected by the PC
	Delay(1000);																				//wait for USB to be connected
	
		
	/* Main program loop */
  while (1)
  {
		distance = (TIM_GetCapture1(TIM2)/72);						//read the TIM2 value and divide by 72 (72Mhz); the result is in us.
		
		sprintf(buf, "%d us\n" , distance);   						//display pulse width in us				
    VCP_PutStr(buf);     

		//MaxSonar EZ0 formula: distance (in inch) = pulse in us * 147
		mm = distance / 147 / 2.54 * 10;									//convert inches to mm

    sprintf(buf, "%.3f mm\n\n" , mm);									//display pulse width in mm
		VCP_PutStr(buf);		
		
    Delay(10);
  }	
}



void MaxSonarEZ0_Init(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;								//structure for GPIO setup
	TIM_ICInitTypeDef  			TIM_ICInitStructure;							//structure for TIM Input Compare
	
	/* ===== Configure the PW (PULSE WIDTH) pin =====
	PB3 will be used as the PW pin in this example.  It will be used in PWM INPUT mode and will use a counter
	tied to a timer to determine the pulse width of the signal from the MaxSonar EZ0 Sonar Range Finder which represent the distance.
	PB3 is shared with TIM2_CH2 and TIM8_CH1 (see the STM32F3-Discovery User Manual p23).  We will use TIM2 in 
	this example, seeing that TIM2 is a general purpose timer (easy to use) and is 32-bits (to hold a large value
	for long pulses).  Also see Chpt 17 on General Purpose Timers (TIM2/3/4) in the STM32F303xx Reference Manual 
	RM0316) for	more detail on the use of TIM2 as PWM INPUT mode.  The TIM2 register value will be updated every
	60ms with a new value for the resultant distance.  A read from the TIM2 register will thus provide the latest
	distance measured.  No software code is required to trigger the sensor and to measure the pulse, it is done
	automatically done continously on the Range Finder and provides a new reading every 50ms
	
	Please note that any other free pin that shares a TIM could be used on the STM32F3-Discovery board.  Just
	make sure that the pin is not used by any of the onboard peripherals (p21-29 in the User Manual).	
	*/
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);				//enable the AHB Peripheral Clock to use GPIOB
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);			//enable the TIM2 clock
  
	/* Pin configuration */
  GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_3;								//configure PB3 in alternate function pushpull mode
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP ;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource3, GPIO_AF_1);			//connect the pin to the desired peripherals' Alternate Function (AF) - TIM2 (GPIO_AF_1)
	
	/* Input Control (IC) configuration - PWM1 Mode configuration: Channel1 */
	TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;					//configre TIM2 for input-PWM mode
  TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
  TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
  TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
  TIM_ICInitStructure.TIM_ICFilter = 0x0;
  TIM_PWMIConfig(TIM2, &TIM_ICInitStructure);

	TIM_SelectInputTrigger(TIM2, TIM_TS_TI2FP2);
  TIM_SelectSlaveMode(TIM2, TIM_SlaveMode_Reset);
  TIM_SelectMasterSlaveMode(TIM2,TIM_MasterSlaveMode_Enable);

  TIM_Cmd(TIM2, ENABLE);																		//enable TIM2
}











// -------------------------------------------------------------------------------

// Function to insert a timing delay of nTime
// ###### DO NOT CHANGE ######
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

// Function to Decrement the TimingDelay variable.
// ###### DO NOT CHANGE ######
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

// Unused functions that have to be included to ensure correct compiling
// ###### DO NOT CHANGE ######
// =======================================================================
uint32_t L3GD20_TIMEOUT_UserCallback(void)
{
  return 0;
}

uint32_t LSM303DLHC_TIMEOUT_UserCallback(void)
{
  return 0;
}
// =======================================================================



