/*#############################################################
File Name       : ex1_Ultrasonic_3pin_vcp.c
Author          : Grant Phillips
Date Modified   : 27/04/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F3-Discovery

Description     : Example program that demonstrates the use
									of the Seeedstudio Ultrasonic Range Sensor 
									(http://www.seeedstudio.com/wiki/Ultra_Sonic_
									range_measurement_module)

									The output is displayed on a PC using the
									Virtual COM Port (VCP).  See the VCP examples
									for an explanation of the use of this feature.
									
Requirements    : * STM32F3-Discovery Board
									* Make sure the VCP_F3.c file is in the 
                    same folder than the project
              
Circuit         : * The Ultrasonic Range Sensor is connected 
										as follows:
										SIG										-	PC0
										VCC										-	5V
										GND										-	GND
									* Mini-USB cable from the User USB port on the
										STM32F3-Discovery to a USB port on a PC
									
See the STM32F3-Discovery User Manual (UM1570) for the block 
diagram of the STM32F303VCT6 processor (p13), a summary of
the GPIO connections (p21-29) and the schematic diagram (p31-34)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "main.h"																								//main library to include for device drivers, peripheral drivers, etc.
#include "VCP_F3.c"																							//for the UART4 functions on the STM32F3-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
uint16_t RANGE3PIN_ReadPulse(void);															//prototype for the user function to read the distance

/* #global variables -----------------------------------------*/
RCC_ClocksTypeDef 			RCC_Clocks;															//structure used for setting up the SysTick Interrupt


// Unused global variables that have to be included to ensure correct compiling */
// ###### DO NOT CHANGE ######
// ===============================================================================
__IO uint32_t TimingDelay = 0;																	//used with the Delay function
__IO uint8_t DataReady = 0;
__IO uint32_t USBConnectTimeOut = 100;
__IO uint32_t UserButtonPressed = 0;
__IO uint8_t PrevXferComplete = 1;
// ===============================================================================



int main(void)
{
	char buf[40];																									//string variable of 40 characters
	uint16_t pulse;            																		//variable for the distance result in us
	uint32_t mm;																									//32-bit unsigned int to hold the distance value in mm
	uint16_t dec, frac;																						//used for converting the integer distance to a float value for printing
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config
					
	/* initialize the Virtual COM Port */
	VCP_ResetPort();																							//pull the USB D+ line low to disconnect the USB device from the PC
	Delay(500);																										//allow a few milliseconds before reconnecting
	VCP_Init();																										//initialize the STM32F3-Discovery as a Virtual COM Port - device gets re-detected by the PC
	Delay(1000);																									//wait for USB to be connected

	/* Main program loop */
  while (1)
  {
		pulse = RANGE3PIN_ReadPulse();            				//get the distance as a us value
		
		//HC-SR04 formula: distance (in m) = pulse in seconds * 340 / 2
		mm = pulse * 340 / 2;															//mm is in micro meters (because distance is in us), e.g. if distance = 728us, then mm = 123760um
		
		dec = mm / 1000;																	//calculate the integer part, e.g. 123
		frac = mm - (dec * 1000);													//calculate the fraction part, e.g. 760

		sprintf(buf, "%d us     %d.%03d mm\n" , pulse, dec, frac);   		//%03d prints a integer value in a field of 3 characters with leading 0's
		VCP_PutStr(buf);
		
    Delay(100);																				//a delay to just prevent the sonar sensor
																											//from being triggered to quickly.  Keep the
																											//delay between triggers above 30ms.
  }	
}


	
/* This function will pulse the Sig pin of the sensor
and then read the width of the result Sig pulse and return
this as a microsecond value. This example uses PC0
for the Sig pin, but can be changed to any other pin.
If more than 1 sonar sensor is used, this function can just
be duplicated using other pins */
uint16_t RANGE3PIN_ReadPulse(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;				//structure used for setting up a GPIO port
  uint32_t counter;            											//counter variable to determine the HIGH pulse
	uint16_t i;

	/* GPIOC Periph clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);		//enable the AHB bus to use GPIOC
	
  /* Configure PC0 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;								
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;							
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;							
  GPIO_Init(GPIOC, &GPIO_InitStructure);						//PC0 is now an output; Sig pin
	       
	/* create a 10us HIGH pulse on Sig pin */
	GPIO_SetBits(GPIOC, GPIO_Pin_0);									//PC0 = 1	
	for(i=0;i<=70;i++)
	{};				//do nothing
  GPIO_ResetBits(GPIOC, GPIO_Pin_0);								//PC0 = 0	

	/* Configure PC0 in input mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;														
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;							
  GPIO_Init(GPIOC, &GPIO_InitStructure);						//PC0 is now an input; Sig pin
		
  counter=0;                     																	//reset the counter
  while(GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_0)==0);         		//wait for the Sig pin to go HIGH
  while(GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_0)==1)            	//increment a counter while the Sig pulse is HIGH
  {
		i=6;																		
		while(i>0)																			//a nominal delay to adjust the counter value to the  																								
			i--;																					//actual pulse width measure on an oscilloscope
		
    counter++;
  }
  return(counter);
}





// -------------------------------------------------------------------------------

// Function to insert a timing delay of nTime
// ###### DO NOT CHANGE ######
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

// Function to Decrement the TimingDelay variable.
// ###### DO NOT CHANGE ######
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

// Unused functions that have to be included to ensure correct compiling
// ###### DO NOT CHANGE ######
// =======================================================================
uint32_t L3GD20_TIMEOUT_UserCallback(void)
{
  return 0;
}

uint32_t LSM303DLHC_TIMEOUT_UserCallback(void)
{
  return 0;
}
// =======================================================================



