/*#############################################################
File Name       : ex1_Zumo_Reflectance_Sensor_Array_vcp.c
Author          : Grant Phillips
Date Modified   : 16/04/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F3-Discovery

Description     : Example program that demonstrates the use
									of the Zumo Reflectance Sensor Array (http:
									//www.pololu.com/product/1419).

									The output is displayed on a PC using the
									Virtual COM Port (VCP).  See the VCP examples
									for an explanation of the use of this feature.
									
Requirements    : * STM32F3-Discovery Board
									* Make sure the VCP_F3.c file is in the 
                    same folder than the project
              
Circuit         : * The Zumo Reflectance Sensor Array is 
										connected as follows:
										(looking at the board on the writing side
										with the sensors at the top and writing the
										right side upward, the pins will be arranged
										as follows:
										1  2  3  4  ...  12 - TOP ROW
										1  2  3  4  ...  12 - BOTTOM ROW)
										
										TOP ROW
										pin2 (SENS2)					-	PB10
										pin4 (SENS1)					-	PB9
										pin6									-	5V
										pin7									-	GND
										pin11(SENS4)					-	PB12
										
										BOTTOM ROW
										pin5 (SENS0)					-	PB8
										pin7									-	GND
										pin8 (SENS3)					-	PB11
										pin11(SENS5)					- PB13
										pin12									-	7.45V voltage source
										
										Make sure the 7.45V source's GND is common to
										the 5V source's GND pin.

									* Mini-USB cable from the User USB port on the
										STM32F3-Discovery to a USB port on a PC
									
See the STM32F3-Discovery User Manual (UM1570) for the block 
diagram of the STM32F303VCT6 processor (p13), a summary of
the GPIO connections (p21-29) and the schematic diagram (p31-34)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "main.h"																								//main library to include for device drivers, peripheral drivers, etc.
#include "VCP_F3.c"																							//for the UART4 functions on the STM32F3-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void InitSystem(void);																					//prototype for the user function to initialize the system

/* #global variables -----------------------------------------*/
RCC_ClocksTypeDef 			RCC_Clocks;															//structure used for setting up the SysTick Interrupt



// Unused global variables that have to be included to ensure correct compiling */
// ###### DO NOT CHANGE ######
// ===============================================================================
__IO uint32_t TimingDelay = 0;																	//used with the Delay function
__IO uint8_t DataReady = 0;
__IO uint32_t USBConnectTimeOut = 100;
__IO uint32_t UserButtonPressed = 0;
__IO uint8_t PrevXferComplete = 1;
// ===============================================================================



int main(void)
{
	char buf[40];																									//string variable of 40 characters
	uint8_t SENS0, SENS1, SENS2, SENS3, SENS4, SENS5;							//variables to hold the value of each sensor (1 or 0)
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config
	
	/* initialize the IO ports and Analogs */
	InitSystem();							
	
	/* initialize the Virtual COM Port */
	VCP_ResetPort();																							//pull the USB D+ line low to disconnect the USB device from the PC
	Delay(500);																										//allow a few milliseconds before reconnecting
	VCP_Init();																										//initialize the STM32F3-Discovery as a Virtual COM Port - device gets re-detected by the PC
	Delay(1000);																									//wait for USB to be connected
	
	/* Main program loop */
  while (1)
  {
		/* read each sensor's status - a 1 indicates BLACK and 0 indicates white */
		SENS0 = GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_8);		//left most sensor
		SENS1 = GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_9);
		SENS2 = GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_10);
		SENS3 = GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_11);
		SENS4 = GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_12);
		SENS5 = GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_13);	//right most sensor
		
		/* display the sensor results */
		sprintf(buf, "%d  %d  %d  %d  %d  %d\n", SENS0, SENS1, SENS2, SENS3, SENS4, SENS5);
		VCP_PutStr(buf);
		
    Delay(200);
  }	
}



void InitSystem(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;											//structure used for setting up a GPIO port
	
	// ================== Initialize the IO Ports ==================
	
	/* GPIOC Periph clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);							//enable the AHB bus to use GPIOB
  
  /* Configure PB13 - PB8 in input with pullup mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_12 | GPIO_Pin_11 | GPIO_Pin_10 | GPIO_Pin_9 | GPIO_Pin_8;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;														
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;							
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;										// enable internal Pullup resistors on the GPIO pins							
  GPIO_Init(GPIOB, &GPIO_InitStructure);			
}





// -------------------------------------------------------------------------------

// Function to insert a timing delay of nTime
// ###### DO NOT CHANGE ######
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

// Function to Decrement the TimingDelay variable.
// ###### DO NOT CHANGE ######
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

// Unused functions that have to be included to ensure correct compiling
// ###### DO NOT CHANGE ######
// =======================================================================
uint32_t L3GD20_TIMEOUT_UserCallback(void)
{
  return 0;
}

uint32_t LSM303DLHC_TIMEOUT_UserCallback(void)
{
  return 0;
}
// =======================================================================



