/*#############################################################
File Name       : ex2_DHT11_measure_interrupt_swv.c
Author          : Grant Phillips
Date Modified   : 19/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F3-Discovery

Description     : Example program that demonstrates the use of
                  a DHT11 Humidity-Temperature Sensor. In this
									example DHT11_Read() is automaticly called
									every second using a TIM17 interrupt.  The
									global variable DHT11_ExitCode is monitored
									to make the last result was valid.									
									
									The output is displayed on a PC using the
									Serial Wire Viewer (SWV).  See the SWV examples
									for an explanation of the use of this feature.

Requirements    : * STM32F3-Discovery Board
									* Make sure the DHT11_F3.c file is in the 
                    same folder than the project
              
Circuit         : * A wire link between PB3 and pin6 of the SWD
										connector (CN3) as explained in the SWV
										examples section
									* The DHT11 sensor connected as:
										1. VCC			-		3V
										2. DATA			-		PB15	(with a 5K6 pull-up 
																		resistor to 3V; pin is 5V tolerant)
										3. NC				-		Not connected
										4. GND			-		GND

See the STM32F3-Discovery User Manual (UM1570) for the block 
diagram of the STM32F303VCT6 processor (p13), a summary of
the GPIO connections (p21-29) and the schematic diagram (p31-34)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "main.h"																								//main library to include for device drivers, peripheral drivers, etc.
#include "DHT11_F3.c"																						//for the DHT11 humidity-temperature sensor functions on the STM32F3-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void TIM17_Config(void);																				//function prototype for a user funtion TIM17_Config()

/* #global variables -----------------------------------------*/



// Unused global variables that have to be included to ensure correct compiling */
// ###### DO NOT CHANGE ######
// ===============================================================================
__IO uint32_t TimingDelay = 0;																	//used with the Delay function
__IO uint8_t DataReady = 0;
__IO uint32_t USBConnectTimeOut = 100;
__IO uint32_t UserButtonPressed = 0;
__IO uint8_t PrevXferComplete = 1;
// ===============================================================================



int main(void)
{
	RCC_ClocksTypeDef 			RCC_Clocks;														//structure used for setting up the SysTick Interrupt
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config

	DHT11_Init();																									//initialize the DHT11 humidity-temperature sensor
	TIM17_Config();																								//initialize TIM14 to generate an interrupt every 1 sec to read from the DHT11
	
	/* Main program loop */
  while (1)
  {
		if(DHT11_ExitCode == DHT11_OK)															//check if last results were OK
		{
			printf("Humidity = %d     Temperature = %d\n", DHT11_Humidity, DHT11_Temperature);	//display the results if result is OK
		}
		else if(DHT11_ExitCode == DHT11_ERROR_TIMEOUT)							//Check if last results caused errors.  These checks could be leaved out
		{																														
			printf("TIMEOUT error\n");
		}
		else if(DHT11_ExitCode == DHT11_ERROR_CHECKSUM)
		{
			printf("CHECKSUM error\n");
		}
  }	
}


/*
Retarget the C library printf function to the SWO Viewer.
Overwrites int fputc(int ch, FILE *f) function used by printf.
*/								
int fputc(int ch, FILE *f)
{
	return(ITM_SendChar(ch));
}



/* Initialize TIM17 to generate an interrupt every 1 sec to measure the pulses per second */
void TIM17_Config(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;													

  /* TIM14 clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM17, ENABLE);

  /* Enable the TIM17 global Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM1_TRG_COM_TIM17_IRQn;	
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Prescaler = 7199;  								//PSC value; meanse that TIM register will be incremented every 100us (72000000 / (7199+1) = 10000Hz = 100us
  TIM_TimeBaseStructure.TIM_Period = 9999;                 		//ARR value - 1; create a time period of 1000000us (100us * (9999 + 1) = 1000000us or 1sec)
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM17, &TIM_TimeBaseStructure);

	/* TIM Interrupts enable */
  TIM_ITConfig(TIM17, TIM_IT_Update, ENABLE);

  /* TIM11 enable counter */
  TIM_Cmd(TIM17, ENABLE);
}



/* Interrupt routine for TIM17 which will be called every 1 second */
void TIM1_TRG_COM_TIM17_IRQHandler(void)
{
	if (TIM_GetITStatus(TIM17, TIM_IT_Update) != RESET)	//Make sure that interrupt flag is set
	{
		DHT11_Read();																			//read from the DHT11 sensor
		TIM_ClearITPendingBit(TIM17, TIM_IT_Update);			//clear interrupt flag
	}
}



// -------------------------------------------------------------------------------

// Function to insert a timing delay of nTime
// ###### DO NOT CHANGE ######
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

// Function to Decrement the TimingDelay variable.
// ###### DO NOT CHANGE ######
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

// Unused functions that have to be included to ensure correct compiling
// ###### DO NOT CHANGE ######
// =======================================================================
uint32_t L3GD20_TIMEOUT_UserCallback(void)
{
  return 0;
}

uint32_t LSM303DLHC_TIMEOUT_UserCallback(void)
{
  return 0;
}
// =======================================================================

